//
//  SLTipOverlay.m
//  demos
//
//  Created by  on 12-3-31.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#import "SLTipOverlay.h"

#define SL_TIP_OVERLAY_DURATION 2
#define SL_TIP_OVERLAY_SHOW_ANIMATION_DURATION 0.6
#define SL_TIP_OVERLAY_DEFAULT_HIDE_DURATION 90
#define SL_TIP_OVERLAY_HEIGHT 36
#define SL_TIP_OVERLAY_SHOWING_CENTER CGPointMake(160,SL_TIP_OVERLAY_HEIGHT/2)

@interface SLTipOverlay()
-(void)setup;
-(void)show;
-(void)hide;
-(void)overHide;
-(void)adjustFrame;
-(IBAction)cancelAction:(id)sender;
-(void)showNextCommand;
-(void)updateWithCommand:(SLTipCommand *)theCommand;
@end
#pragma mark - SLTipOverlay
@implementation SLTipOverlay
@synthesize showing,delegate,command,offset,parentController,isContinuous;
#pragma mark views
-(void)setup{
    self.backgroundColor = [UIColor clearColor];
    UIView *bg = [[UIView alloc] initWithFrame:self.bounds];
    bg.backgroundColor = [UIColor lightGrayColor];
    bg.alpha = 0.8;
    [self addSubview:bg];
    [bg release];
    
    msgLabel = [[UILabel alloc] initWithFrame:CGRectMake(30, 0, 320-30-30, self.frame.size.height)];
    msgLabel.backgroundColor = [UIColor clearColor];
    msgLabel.textAlignment = UITextAlignmentCenter;
    msgLabel.textColor = [UIColor whiteColor];
    msgLabel.font = [UIFont systemFontOfSize:14];
    msgLabel.lineBreakMode = UILineBreakModeMiddleTruncation;
    msgLabel.adjustsFontSizeToFitWidth = YES;
    msgLabel.minimumFontSize = 7;
    [self addSubview:msgLabel];
    
    UIButton *btn = [UIButton buttonWithType:UIButtonTypeContactAdd];
    btn.frame = CGRectMake(320-30, 0, 30, self.frame.size.height);
    btn.transform = CGAffineTransformMakeRotation(M_PI_4);
    [self addSubview:btn];
    [btn addTarget:self action:@selector(cancelAction:) forControlEvents:UIControlEventTouchUpInside];
    closeBtn = [btn retain];
    
    loadingView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"second"]];
    loadingView.frame = CGRectMake(0, 0, 30, self.frame.size.height);
    [self addSubview:loadingView];
    
//    UIActivityIndicatorView *aview = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhite];
//    CGRect framee = aview.frame;
//    framee.origin.x = 50;
//    framee.size.width = 30;
//    framee.size.height = 30;
//    aview.frame = framee;
//    [aview startAnimating];
//    [self addSubview:aview];
}
-(void)updateViewsWithCommand:(SLTipCommand *)theCommand{
    if (theCommand) {
        msgLabel.text = theCommand.msg;
        if (theCommand.loadingProcess < 0) {
            [self invalidateLoadingTimer];
            loadingView.hidden = YES;
        }else {
            loadingView.hidden = NO;
            if (theCommand.loadingProcess >= 1) {
                [self invalidateLoadingTimer];
                loadingView.transform = CGAffineTransformIdentity;
                loadingView.image = [UIImage imageNamed:@"first"];
            }else {
                loadingView.image = [UIImage imageNamed:@"second"];
                [self fireLoadingTimer];
            }
        }
        closeBtn.hidden = !theCommand.closeable;
    }
}
-(void)adjustFrame{
    showingCenter = SL_TIP_OVERLAY_SHOWING_CENTER;
    if (![UIApplication sharedApplication].statusBarHidden) {
        showingCenter.y += 20;
    }
    if (self.parentController && !self.parentController.navigationBarHidden) {
        showingCenter.y += 44;
    }
    showingCenter.y += self.offset;
    self.center = CGPointMake(showingCenter.x, showingCenter.y - SL_TIP_OVERLAY_HEIGHT);
}

-(void)rotateLoadingView{
    loadingView.transform = CGAffineTransformRotate(loadingView.transform, 0.5);
}
-(id)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    if (self) {
        [self setup];
        commandQueue = [[NSMutableArray array] retain];
        self.isContinuous = YES;
    }
    return self;
}
-(void)layoutSubviews{
//    NSLog(@"%d",self.parentController.navigationBarHidden);
}
#pragma mark command
-(void)removeFinishedCommand{
    @synchronized(commandQueue){
        [commandQueue removeLastObject];
    }
}
-(void)clearAllCommands{
    @synchronized(commandQueue){
        [commandQueue removeAllObjects];
    }
}
-(void)setCommand:(SLTipCommand *)newCommand{
    if (self.command) {
        [command release];
    }
    command = [newCommand retain];
}
#pragma mark timer
-(void)invalidateLoadingTimer{
    isLoading = NO;
//    if (loadingTimer) {
//        [loadingTimer invalidate];
//        [loadingTimer release];
//        loadingTimer = nil;
//    }
}
-(void)setLoadingViewTransformIdentity{
    loadingView.transform = CGAffineTransformIdentity;
}
-(void)startLoadingAnimation{
    isLoading = YES;
    while (isLoading) {
        [self performSelectorOnMainThread:@selector(rotateLoadingView) withObject:nil waitUntilDone:NO];
        usleep(50000);
    }
    [self performSelectorOnMainThread:@selector(setLoadingViewTransformIdentity) withObject:nil waitUntilDone:NO];
}
-(void)fireLoadingTimer{
    [self invalidateLoadingTimer];
    [NSThread detachNewThreadSelector:@selector(startLoadingAnimation) toTarget:self withObject:nil];
//    loadingTimer = [[NSTimer scheduledTimerWithTimeInterval:0.1 target:self selector:@selector(rotateLoadingView) userInfo:nil repeats:YES] retain];
}
-(void)invalidateHideTimer{
    if (hideTimer) {
        [hideTimer invalidate];
        [hideTimer release];
        hideTimer = nil;
    }
}
-(void)fireHideTimerWithCommand:(SLTipCommand *)theCommand{
    if (theCommand) {
        [self invalidateHideTimer];
        if (theCommand.autoHide) {
            hideTimer = [[NSTimer scheduledTimerWithTimeInterval:theCommand.duration target:self selector:@selector(hide) userInfo:nil repeats:NO] retain];
        }else {
            hideTimer = [[NSTimer scheduledTimerWithTimeInterval:SL_TIP_OVERLAY_DEFAULT_HIDE_DURATION target:self selector:@selector(hide) userInfo:nil repeats:NO] retain];
        }
    }
}
#pragma mark animation
-(void)showNext{
    [UIView beginAnimations:@"animationTurnToNext" context:nil];
    [UIView setAnimationDuration:0.5f];
    [UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
    [UIView setAnimationRepeatAutoreverses:NO];
    [UIView setAnimationTransition:UIViewAnimationTransitionCurlDown forView:self cache:YES];
    [self updateWithCommand:self.command];
    [UIView commitAnimations];
}
-(void)overShow{
    [self fireHideTimerWithCommand:self.command];
}
-(void)show{
    if (self.command) {
        [self updateViewsWithCommand:self.command];
        [UIView setAnimationsEnabled:YES];
        [UIView beginAnimations:@"animateSHOW" context:nil];
        [UIView setAnimationDuration:SL_TIP_OVERLAY_SHOW_ANIMATION_DURATION];
        [UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
        [UIView setAnimationDidStopSelector:@selector(overShow)];
        [UIView setAnimationDelegate:self];
        self.center = showingCenter;
        [UIView commitAnimations];
    }
}
-(void)overHide{
//    [UIView setAnimationsEnabled:NO];
    [self invalidateLoadingTimer];
    msgLabel.text = nil;
    self.showing = NO;
    if (self.delegate && [self.delegate respondsToSelector:@selector(tipOverlay:DidHideWithCommand:)]) {
        [self.delegate tipOverlay:self DidHideWithCommand:self.command];
    }
    [self removeFinishedCommand];
    [self showNextCommand];
}
-(void)hide{
    if (self.isContinuous && commandQueue.count > 1) {
        [self invalidateLoadingTimer];
        [self removeFinishedCommand];
        @synchronized(commandQueue){
            self.command = commandQueue.lastObject;
        }
        [self showNext];
    }else {
        BOOL shouldHide = YES;
        if (self.delegate && [self.delegate respondsToSelector:@selector(tipOverlay:ShouldHideWithCommand:)]) {
            shouldHide = [self.delegate tipOverlay:self ShouldHideWithCommand:self.command];
        }
        if (shouldHide) {
            [UIView beginAnimations:@"animateHIDE" context:nil];
            [UIView setAnimationDuration:SL_TIP_OVERLAY_SHOW_ANIMATION_DURATION];
            [UIView setAnimationCurve:UIViewAnimationCurveEaseOut];
            [UIView setAnimationDidStopSelector:@selector(overHide)];
            [UIView setAnimationDelegate:self];
            self.center = CGPointMake(showingCenter.x,-showingCenter.y);
            [UIView commitAnimations];
        }
    }
}
#pragma mark actions
-(IBAction)cancelAction:(id)sender{
    [self invalidateHideTimer];
    if (self.delegate && [self.delegate respondsToSelector:@selector(tipOverlay:ShouldClearCommandQueueOnCanceledWithCommand:)]) {
        BOOL isClear = [self.delegate tipOverlay:self ShouldClearCommandQueueOnCanceledWithCommand:self.command];
        if (isClear) {
            [self clearAllCommands];
        }
    }
    [self hide];
}
-(void)setParentController:(UINavigationController *)newParentController{
    if (self.parentController == newParentController) {
        return;
    }
    if (self.parentController) {
        [parentController release];
    }
    [self removeFromSuperview];
    [self invalidateHideTimer];
    [self invalidateLoadingTimer];
    [self clearAllCommands];
    self.showing = NO;
    parentController = [newParentController retain];
}
-(void)showNextCommand{
    if (!self.isShowing && commandQueue.count > 0 && self.parentController) {
        @synchronized(commandQueue){
            self.showing = YES;
            self.command = commandQueue.lastObject;
            [self removeFromSuperview];
            [self adjustFrame];
            [self.parentController.view insertSubview:self belowSubview:self.parentController.navigationBar];
            [self show];
        }
    }
}
-(void)updateWithCommand:(SLTipCommand *)theCommand{
    if (theCommand) {
        [self invalidateHideTimer];
        [self updateViewsWithCommand:theCommand];
        [self fireHideTimerWithCommand:theCommand];
    }
}
-(void)dealloc{
    [msgLabel release];
    [closeBtn release];
    [loadingView release];
    [command release];
    [hideTimer release];
    [commandQueue release];
    [parentController release];
    [super dealloc];
}
#pragma mark - show methods
-(void)showCommand:(SLTipCommand *)theCommand{
    if (!theCommand) {
        return;
    }
    if (self.isShowing && theCommand.loadingProcess >= 0){
        @synchronized(commandQueue){
            for (SLTipCommand *com in commandQueue) {
                if (com.loadingProcess >= 0 && com.tag == theCommand.tag) {
                    if (com == self.command) {
                        [self updateWithCommand:theCommand];
                    }else {
                        [commandQueue replaceObjectAtIndex:[commandQueue indexOfObject:com] withObject:theCommand];
                    }
                    return;
                }
            }
            
        }
    }
    [commandQueue insertObject:theCommand atIndex:0];
    [self showNextCommand];
}

#pragma mark - shared instance
+(SLTipOverlay *)sharedTipInController:(UINavigationController *)controller{
    static SLTipOverlay *_tipOverlay;
    if (!_tipOverlay) {
        _tipOverlay = [[SLTipOverlay alloc] initWithFrame:CGRectMake(0, -SL_TIP_OVERLAY_HEIGHT, 320, SL_TIP_OVERLAY_HEIGHT)];
    }
    _tipOverlay.parentController = controller;
    return _tipOverlay;
}
@end

#pragma mark - SLTimCommand
@implementation SLTipCommand
@synthesize loadingProcess, closeable,msg,tag,autoHide,duration;

-(id)initWithMessage:(NSString *)message Closeable:(BOOL)isCloseable LoadingProcess:(CGFloat)process AutoHide:(BOOL)isAutoHide Duration:(CGFloat)hideDuration{
    self = [super init];
    if (self) {
        self.msg = message;
        self.closeable = isCloseable;
        self.loadingProcess = process;
        self.autoHide = isAutoHide;
        self.duration = hideDuration;
    }
    return self;
}
-(void)dealloc{
    [msg release];
    [super dealloc];
}
//return tip command with message,and closeable will be true and loadingProcess will be -1 as default
+(SLTipCommand *)tipCommandForMessage:(NSString *)message{
    return [[[SLTipCommand alloc] initWithMessage:message Closeable:YES LoadingProcess:-1 AutoHide:YES Duration:SL_TIP_OVERLAY_DURATION ] autorelease];
}
//return tip command with message and process,the closeable will be false as default
+(SLTipCommand *)tipCommandForLoadingWithMessage:(NSString *)message LoadingProcess:(CGFloat)process{
    return [[[SLTipCommand alloc] initWithMessage:message Closeable:NO LoadingProcess:process AutoHide:NO Duration:0] autorelease];
}
//return tip command with message and process,the closeable will be false,autohide will be true as default
+(SLTipCommand *)tipCommandForLoadingFinishedWithMessage:(NSString *)message LoadingProcess:(CGFloat)process{
    return [[[SLTipCommand alloc] initWithMessage:message Closeable:NO LoadingProcess:process AutoHide:YES Duration:SL_TIP_OVERLAY_DURATION] autorelease];
}

@end
